package net.chrisrichardson.foodToGo.order;

import java.util.*;

/**
 * The opening hours of the restaurant.
 * Converts between the database representation and the java representation
 * 
 * In the database an opening/closing time is stored as a number:
 *          dayofweekHourMinute
 * e.g. 12015 is Monday at 8.15 pm (20=24 hours clock)
 * A restaurant currently has seven pairs of (opening, closing) times
 */

public class OpeningHours {

	private HashMap hours = new HashMap();

	public OpeningHours() {
	}

	public OpeningHours(
		int open1,
		int close1,
		int open2,
		int close2,
		int open3,
		int close3,
		int open4,
		int close4,
		int open5,
		int close5,
		int open6,
		int close6,
		int open7,
		int close7) {
		add(open1, close1);
		add(open2, close2);
		add(open3, close3);
		add(open4, close4);
		add(open5, close5);
		add(open6, close6);
		add(open7, close7);
	}

	/**
	 * Adds the time range to the opening hours
	 */

	private void add(int open, int close) {
		if (open == 0)
			return;

		Date openDate = makeDate(open);
		Date closeDate = makeDate(close);
		int dayOfWeek = makeDayOfWeek(open);
		TimeRange tr = new TimeRange(openDate, closeDate);
		System.out.println(
			"open = "
				+ open
				+ ", close = "
				+ close
				+ ", dayOfWeek="
				+ dayOfWeek
				+ ", tr= "
				+ tr);
		add(dayOfWeek, tr);
	}

	/**
	 * Converts a database time to a Date
	 */

	private Date makeDate(int open) {
		int hour = (open % (100 * 100)) / 100;
		int minute = open % 100;
		Calendar c = Calendar.getInstance();
		c.clear();
		c.set(Calendar.HOUR_OF_DAY, hour);
		c.set(Calendar.MINUTE, minute);
		return c.getTime();
	}

	/*
	 * Returns the day or the week
	 */

	private int makeDayOfWeek(int open) {
		return open / (100 * 100);
	}

	/*
	 * Adds it to a table
	 */

	public void add(int dayOfWeek, TimeRange range) {
		Integer key = new Integer(dayOfWeek);
		ArrayList times = (ArrayList) hours.get(key);
		if (times == null) {
			times = new ArrayList();
			hours.put(key, times);
		}
		times.add(range);
	}

	/*
	 * Determines whether a restaurant is open at a particular time
	 */

	public boolean isOpenAtThisTime(Date date) {
		Calendar c = Calendar.getInstance();
		c.setTime(date);
		int dayOfWeek = c.get(Calendar.DAY_OF_WEEK);

		ArrayList times = (ArrayList) hours.get(new Integer(dayOfWeek));
		if (times == null)
			return false;

		for (Iterator it = times.iterator(); it.hasNext();) {
			TimeRange range = (TimeRange) it.next();
			if (range.isOpenAtThisTime(date))
				return true;
		}

		return false;
	}

	public Iterator getOpeningTimes() {
		return hours.entrySet().iterator();
	}

    /**
     * Convert the opening hours into a sequence of integers: open1, close1, open2, close2...
     */
    
	public Iterator getOpeningTimesAsInts() {
		Collection result = new ArrayList();
		for (Iterator it = getOpeningTimes(); it.hasNext();) {
			Map.Entry entry = (Map.Entry) it.next();
			Integer dayOfWeek = (Integer) entry.getKey();
			ArrayList times = (ArrayList) entry.getValue();
			for (Iterator it2 = times.iterator(); it2.hasNext();) {
				TimeRange tr = (TimeRange) it2.next();
				Date open = tr.getOpenTime();
				Date close = tr.getCloseTime();
				int intOpen = makeIntTime(dayOfWeek.intValue(), open);
				int intClose = makeIntTime(dayOfWeek.intValue(), close);
                result.add(new Integer(intOpen));
                result.add(new Integer(intClose));
			}
		}
		return result.iterator();
	}

     /**
     * Converts from the internal/Java representation
     * to the database representation
     * 
     * @param dayOfWeek
     * @param timeOfDay
     * @return 
     */
    private int makeIntTime(int dayOfWeek, Date timeOfDay) {
        Calendar c = Calendar.getInstance();
        c.setTime(timeOfDay);
        return dayOfWeek * (100 * 100)
            + (c.get(Calendar.HOUR_OF_DAY) * 100)
            + c.get(Calendar.MINUTE);
    }
}
