package net.chrisrichardson.foodToGo.order;



import java.util.*;
import javax.jdo.*;
import net.chrisrichardson.foodToGo.order.*;
import net.chrisrichardson.foodToGo.util.*;


public class Restaurant implements InstanceCallbacks {
	// private Integer id;
	private String name;
	private Set serviceArea;
	private String notificationEmailAddress;
	private List menuItems;
	private OpeningHours openingHours;

	// These fields store the opening hours of the restaurant
	// Denormalized

	private int open1;
	private int close1;
	private int open2;
	private int close2;
	private int open3;
	private int close3;
	private int open4;
	private int close4;
	private int open5;
	private int close5;
	private int open6;
	private int close6;
	private int open7;
	private int close7;

    public boolean isInServiceArea(Address address, Date deliveryTime) {
        return isInServiceArea(address) && isOpenAtThisTime(deliveryTime);
    }

    private boolean isOpenAtThisTime(Date deliveryTime) {
        return getOpeningHours().isOpenAtThisTime(deliveryTime);
    }




	/*
	* Creates a Restaurant
	*/

	public Restaurant(
		String name,
		String zipCode,
		OpeningHours openingHours,
		Collection menuItems) {
		this.name = name;
		this.serviceArea = new HashSet();
		this.serviceArea.add(zipCode);
		this.openingHours = openingHours;
		this.menuItems = new ArrayList();

		for (Iterator it = menuItems.iterator(); it.hasNext();) {
			MenuItem menuItem = (MenuItem) it.next();
			this.menuItems.add(menuItem);
		}
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Returns true of the address and time are served 
	 * by the restaurant
	 * 
	 * @param address
	 * @return 
	 */
	public boolean isInServiceArea(Address address) {
		return serviceArea.contains(address.getZip());
	}

	/**
	 * Returns the menu items of the restaurant
	 * 
	 * @return 
	 */
	public MenuItemCollection findMenuItems() {
		return new MenuItemCollection(menuItems);
	}

	public String getNotificationEmailAddress() {
		return notificationEmailAddress;
	}

	public void setNotificationEmailAddress(String notificationEmailAddress) {
		this.notificationEmailAddress = notificationEmailAddress;
	}

	public Set getServiceArea() {
		return serviceArea;
	}

	public void setServiceArea(Set serviceArea) {
		this.serviceArea = serviceArea;
	}

	class MenuItemComparator implements Comparator {

		/**
		 * Returns the sorted menu items
		 * 
		 * @param arg0
		 * @param arg1
		 * @return 
		 */
		public int compare(Object arg0, Object arg1) {
			int x = ((MenuItem) arg0).getIndex();
			int y = ((MenuItem) arg1).getIndex();
			if (x < y)
				return -1;
			if (x == y)
				return 0;
			return 1;
		}

	}

	List getMenuItems() {
		TreeSet set = new TreeSet(new MenuItemComparator());
		set.addAll(menuItems);
		ArrayList result = new ArrayList();
		result.addAll(set);
		return result;
	}

	void setMenuItems(List menuItems) {
		this.menuItems = menuItems;
	}

	protected OpeningHours getOpeningHours() {
		// I need to touch a field in order to cause jdoPostLoad() to be invoked
		int x = open1;
		return openingHours;
	}

	protected OpeningHours makeOpenHours() {
		OpeningHours openingHours =
			new OpeningHours(
				open1,
				close1,
				open2,
				close2,
				open3,
				close3,
				open4,
				close4,
				open5,
				close5,
				open6,
				close6,
				open7,
				close7);
		return openingHours;
	}

    /**
     * Create the openingHours
     */
    
	public void jdoPostLoad() {
		System.out.println("jdoPostLoad");
		openingHours = makeOpenHours();
	}

	public void jdoPreClear() {
		System.out.println("jdoPreClear");
	}

	public void jdoPreDelete() {
		System.out.println("jdoPreDelete");
	}

    /**
     * Flatten the opening Hours
     */
    
	public void jdoPreStore() {
		System.out.println("jdoPreStore");
		if (openingHours == null) {
			open1 = 0;
			close1 = 0;
			open2 = 0;
			close2 = 0;
			open3 = 0;
			close3 = 0;
			open4 = 0;
			close4 = 0;
			open5 = 0;
			close5 = 0;
			open6 = 0;
			close6 = 0;
			open7 = 0;
			close7 = 0;
		} else {

			int i = 1;
			for (Iterator it = openingHours.getOpeningTimesAsInts(); it.hasNext();) {
				Integer open = (Integer) it.next();
				Integer close = (Integer) it.next();

				int intOpen = open.intValue();
				int intClose = close.intValue();
				switch (i) {
					case 1 :
						open1 = intOpen;
						close1 = intClose;
						break;
					case 2 :
						open2 = intOpen;
						close2 = intClose;
						break;
					case 3 :
						open3 = intOpen;
						close3 = intClose;
						break;
					case 4 :
						open4 = intOpen;
						close4 = intClose;
						break;
					case 5 :
						open5 = intOpen;
						close5 = intClose;
						break;
					case 6 :
						open6 = intOpen;
						close6 = intClose;
						break;
					case 7 :
						open7 = intOpen;
						close7 = intClose;
						break;
					default :
						throw new NotYetImplementedException("i=" + i);
				}
				i++;
			}
		}
	}

}
