package net.chrisrichardson.foodToGo.order.jdoImpl;

import java.util.*;

import javax.jdo.*;

import net.chrisrichardson.foodToGo.order.*;
import net.chrisrichardson.foodToGo.orderService.values.*;
import net.chrisrichardson.foodToGo.util.*;

public class JDORestaurantRepositoryImpl implements RestaurantRepositoryImpl {

    public RestaurantValuePagedCollection getRestaurantsThatServeZipCodeAndTime(
        Address deliveryAddress,
        Date deliveryTime) {
        Collection result = findRestaurants(deliveryAddress, deliveryTime);

        RestaurantValueCollection restaurants = new RestaurantValueCollection();
        PersistenceManager pm = PMRegistry.getPM();

        for (Iterator it = result.iterator(); it.hasNext();) {
            Restaurant r = (Restaurant) it.next();
            restaurants.add(
                new RestaurantValue(getObjectId(r), r.getName()));
        }
        return new RestaurantValuePagedCollection(
            restaurants.size(),
            1,
            restaurants.size(),
            restaurants);
    }

    private Collection findRestaurants(Address deliveryAddress, Date deliveryTime) {
        int timeOfDay = makeTimeOfDay(deliveryTime);

        PersistenceManager pm = PMRegistry.getPM();
		String queryString = makeQueryString();
        Query query = pm.newQuery(Restaurant.class, queryString);
        query.declareParameters("String zipCode, int timeOfDay");
        query.compile();
        Collection result =
            (Collection) query.executeWithArray(
                new Object[] { deliveryAddress.getZip(), new Integer(timeOfDay)});
        return result;
    }

	private String makeQueryString() {
		String param = "timeOfDay";
		String queryString =
		    "serviceArea.contains(zipCode) && (" + makeTimeOfDayQuery(param) + ")";
		System.out.println("QueryString = " + queryString);
		return queryString;
	}

    private String makeTimeOfDayQuery(String param) {
        StringBuffer query = new StringBuffer();
        for (int i = 1; i < 8; i++) {
            String s =
                "(open" + i + " <= " + param + " && close" + i + " > " + param + " )";
            if (i != 1)
                query.append(" || ");
            query.append(s);
        }
        return query.toString();
    }

    private int makeTimeOfDay(Date deliveryTime) {
        Calendar c = Calendar.getInstance();
        c.setTime(deliveryTime);

        int dayOfWeek = c.get(Calendar.DAY_OF_WEEK);
        int hour = c.get(Calendar.HOUR_OF_DAY);
        int minute = c.get(Calendar.MINUTE);
        return minute + 100 * (hour + (100 * dayOfWeek));
    }

    public boolean isRestaurantAvailable(Address deliveryAddress, Date deliveryDate) {
        // TODO - more efficient way?
        return !findRestaurants(deliveryAddress, deliveryDate).isEmpty();
    }

	public Restaurant findRestaurant(String restaurantId) {
		PersistenceManager pm = PMRegistry.getPM();
		return (Restaurant) pm.getObjectById(
			pm.newObjectIdInstance(Restaurant.class, restaurantId),
			true);
	}

    public String getObjectId(Restaurant object) {
        return PMRegistry.getPM().getObjectId(object).toString();
    }

    public String getObjectId(MenuItem object) {
        return PMRegistry.getPM().getObjectId(object).toString();
    }

}
